///////////////////////////////////////////////////////////////////////////////
// GrabBagGame
//
// The end of the game is generally reached with finding and holding
// all 10 flavins. Flavins make you more powerful, the more you have.
// Kills don't go toward your score, only collecting flavins/bags do.
//
///////////////////////////////////////////////////////////////////////////////
class GoldenGunGame extends DeathMatch
	config;

var int BagCheckTime;

const BAG_CHECK_TIME		=	5.0;
const BAG_CHECK_ERROR_TIME	=	2.0; // check more often when there's a possible error

///////////////////////////////////////////////////////////////////////////////
// Only count kills, score doesn't factor in kills
///////////////////////////////////////////////////////////////////////////////
function ScoreKill(Controller Killer, Controller Other)
{
	// Don't score suicides as anything (don't even log them as kills)
    if( (killer == Other) || (killer == None) )
	{
    	if (Other!=None)
        {
			//Other.PlayerReplicationInfo.Score -= 1;
			//ScoreEvent(Other.PlayerReplicationInfo,-1,"self_frag");
        }
	}
    else if ( killer.PlayerReplicationInfo != None )
	{
		/*Log ("Other.Pawn was" @ Other.Pawn);
		//Log ("P2Pawn(Other.Pawn) was " @ P2Pawn(Other.Pawn));
		//Log ("DyingDamageType was" @ P2Pawn(Other.Pawn).DyingDamageType);
		//Log ("LastDamageType was" @ P2Pawn(Other.Pawn).LastDamageType);*/
		if (P2Pawn(Other.Pawn).LastDamageType == class'GoldenBulletDamage')
		{
			Killer.PlayerReplicationInfo.Score += 1;
			ScoreEvent(Killer.PlayerReplicationInfo,1,"frag");
		}
		Killer.PlayerReplicationInfo.Kills++;
	}

	if ( GameRulesModifiers != None )
		GameRulesModifiers.ScoreKill(Killer, Other);

    if ( (Killer != None) || (MaxLives > 0) )
		CheckScore(Killer.PlayerReplicationInfo);
}

///////////////////////////////////////////////////////////////////////////////
// Check to make sure the number of bags in the level are equal to the number
// of original pickups placed in the level--should be 10 at all times.
///////////////////////////////////////////////////////////////////////////////
function MaintainBagCount()
{
	local PathNode pn;
	local PlayerStart ps;
	local GoldenGunPickup ggp;
	local GoldenGunWeapon ggw;
	local GoldenGunWeaponSS ggw2;
	local int NumFlavins, Countdown;
	local bool ggPlaced;
	local PlayerReplicationInfo pri;

	local array<vector> GunSpots;

	// only update every so many seconds
	BagCheckTime += Level.TimeDilation;

	if(BagCheckTime > BAG_CHECK_TIME)
	{
		BagCheckTime=0; // reset
		ggPlaced=false;
		// check to see goldengun is placed yet
		foreach DynamicActors(class'GoldenGunPickup', ggp)
		{
			ggPlaced=true;
		}
		if (!ggPlaced) {
			// check for player inventories
			foreach DynamicActors(class'GoldenGunWeapon', ggw)
			{
				ggPlaced = true;
			}
			foreach DynamicActors(class'GoldenGunWeaponSS', ggw2)
			{
				ggPlaced = true;
			}
		}
		// Check all flavins in the level
		if (!ggPlaced) {	
		// Still can't find golden gun, put it on the level somewhere
			//log("Attempting to place golden gun.");
			if (GunSpots.Length > 0)
				GunSpots.Remove(0, GunSpots.Length);
			NumFlavins = 0;
			foreach AllActors(class'PathNode', pn) {
				GunSpots.Insert(GunSpots.Length, 1.0);
				GunSpots[GunSpots.Length-1] = pn.Location;
				NumFlavins++;
				//log("Added PathNode " @ pn @ " to gun spot list");
			}
			foreach AllActors(class'PlayerStart', ps) {
				GunSpots.Insert(GunSpots.Length, 1.0);
				GunSpots[GunSpots.Length-1] = ps.Location;
				NumFlavins++;
				//log("Added PlayerStart " @ ps @ " to gun spot list");
			}
			Countdown=rand(NumFlavins);
			ggp = spawn(class'GoldenGunPickup',,,GunSpots[Countdown]);
			ggp.SetPhysics(PHYS_FALLING);
			ggp.bAllowedToRespawn=false;
			ggp.RespawnTime=0;
			BroadcastLocalizedMessage(class'GunMessages', 2, None, None, None );
			//log("Spawned the golden gun.");
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Match in progress
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
State MatchInProgress
{
	function Timer()
	{
		MaintainBagCount();
		Super.Timer();
	}

}

defaultproperties
{
	GameName="Golden Gun"
	BeaconName="GG"
	GoalScore=6
	MapNameGameCode="d"	// must be lower case!
	bPlayersMustBeReady=true
	// Grab Bag Menu Settings Type
	//SettingsMenuType="UTBrowser.UTGBSettingsSClient";

	HUDType="MultiGame.MpHUD"
	//HUDType="MultiGame.GBHUD"
	ScoreBoardType="MultiGame.DMScoreBoard"

	MapListType="MultiGame.GBMapList"
	GameReplicationInfoClass=Class'MultiBase.MpGameReplicationInfo'
	DeathMessageClass=class'MultiGame.xDeathMessage'
	MutatorClass="MultiGame.xMutator"
	LevelRulesClass=class'LevelGamePlay'

	GameBaseEquipment[0]=(weapclass=class'Inventory.UrethraWeapon')
	GameBaseEquipment[1]=(weapclass=class'Inventory.FootWeapon')
	GameBaseEquipment[2]=(weapclass=class'Inventory.BatonWeapon')
	GameBaseEquipment[3]=(weapclass=class'Inventory.PistolWeapon')
	//GameBaseEquipment[4]=(weapclass=class'kMods.kGGRadarInv')
}
