class MutMassacre extends xMutator
	config(kMods);

var array<Pawn> BystanderArray;
var array<vector> GunSpots;
var int NumBystanders;

var config class<Pawn> BystanderClass;
var config int MinBystanders;
var config int MaxBystanders;

static function class<Pawn> GetBystanderClass()
{
	return Default.BystanderClass;
}

function PostBeginPlay()
{
	SetupGunSpots();
	Level.Game.GameRulesModifiers.AddGameRules(spawn(class'MassacreRules'));
	NumBystanders=Rand(MaxBystanders-MinBystanders+1)+MinBystanders;
	//log("NumBystanders was" @ NumBystanders);
}

function SetupGunSpots()
{
	local PathNode pn;
	local PlayerStart ps;
	local int NumFlavins, Countdown;

	if (GunSpots.Length > 0)
		GunSpots.Remove(0, GunSpots.Length);
	NumFlavins = 0;
	// Look first for PathNodes with the "AirMail" tag
	foreach AllActors(class'PathNode', pn) {
		if (Instr(pn.tag,"massacre") >= 0)
		{
			GunSpots.Insert(GunSpots.Length, 1.0);
			GunSpots[GunSpots.Length-1] = pn.Location;
			NumFlavins++;
		}
		//log("Added PathNode " @ pn @ " to gun spot list");
	}
	if (NumFlavins == 0) { // No AMN's found, use regular nodes instead.
		// Adds all pathnodes to the list
		foreach AllActors(class'PathNode', pn) {
			GunSpots.Insert(GunSpots.Length, 1.0);
			GunSpots[GunSpots.Length-1] = pn.Location;
			NumFlavins++;
			//log("Added PathNode " @ pn @ " to gun spot list");
		}
		// Adds all playerstarts to the list
		foreach AllActors(class'PlayerStart', ps) {
			GunSpots.Insert(GunSpots.Length, 1.0);
			GunSpots[GunSpots.Length-1] = ps.Location;
			NumFlavins++;
			//log("Added PlayerStart " @ ps @ " to gun spot list");
		}
	}
}

event Tick(float DeltaTime)
{
	local Pawn P;
	local Controller C;
	local int i;

	if (BystanderArray.Length < NumBystanders)
	{
		//log("Adding" @ NumBystanders-BystanderArray.Length @ " bystanders to the bystanderarray");
		BystanderArray.Insert(0,NumBystanders-BystanderArray.Length);
		//log("Bystander array length is now" @ BystanderArray.Length);
	}

	for (i=0;i<BystanderArray.Length;i++)
		if (BystanderArray[i] == None)
		{
			P = spawn(BystanderClass,,,GunSpots[Rand(GunSpots.Length)]);
			if (P != None)
			{
				BystanderArray[i] = P;
				spawn(class'kMassacreTeleport',,,P.Location);
				if (P.ControllerClass != None)
				{
					C = spawn(P.ControllerClass);
					if (C != None)
					{
						C.Possess(P);
						if (PersonController(C) != None)
							C.GotoState('Thinking');
					}
				}
			}
		}
}

defaultproperties
{
	GroupName="Bystanders"
	FriendlyName="Massacre"
	Description="Inserts bystanders into deathmatch maps. Killing bystanders can either add points, subtract points, or have no effect on your score depending on the configuration."

	BystanderClass=class'BasePeople.Bystanders'
	MinBystanders=10
	MaxBystanders=20
}
