///////////////////////////////////////////////////////////////////////////////
// LauncherWeapon
// Copyright 2002 Running With Scissors, Inc.  All Rights Reserved.
//
// Rocket launcher
//
// Charged weapon. Needs to fuel each rocket. 
// If fired before it reaches the red in the fuel, it's a normal, forward
// thrusting rocket.
// If it reaches the red point and is fired, it shoots a seeking rocket,
// which targets the highest threat/ then nearest character.
// Alt-fire just is the same but doesn't shoot a seeking rocket at the end.
//
///////////////////////////////////////////////////////////////////////////////

class kNukeMPWeapon extends LauncherWeapon;

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
function Notify_ShootLauncher()
{
	local vector StartTrace, X,Y,Z, markerpos, HitNormal, HitLocation;
	local actor HitActor;
	local kNukeMPProjectile lpro;
	local kNukeMPSeekingProjectileTrad lspro;
	local float giveback;
	
	if(AmmoType != None)
	{
		GetAxes(Instigator.GetViewRotation(),X,Y,Z);
		StartTrace = GetFireStart(X,Y,Z); 
		AdjustedAim = Instigator.AdjustAim(AmmoType, StartTrace, 2*AimError);
		CalcAIChargeTime();
		ShootStyleChanger=0;
		// Make sure we're not generating this on the other side of a thin wall
		//if(FastTrace(Instigator.Location, StartTrace))
		HitActor = Trace(HitLocation, HitNormal, Instigator.Location, StartTrace, true);
		if(HitActor == None
			|| (!HitActor.bStatic
				&& !HitActor.bWorldGeometry))
		{
			if(!bSeeking)
				// fire a normal rocket
			{
				lpro = spawn(class'kNukeMPProjectile',Instigator,,StartTrace, AdjustedAim);
				if(lpro != None)
				{
					lpro.Instigator = Instigator;
					// Compensate for catnip time, if necessary. Don't do this for NPCs
					if(FPSPawn(Instigator) != None
						&& FPSPawn(Instigator).bPlayer)
						ChargeTime /= Level.TimeDilation;
					lpro.SetupShot(ChargeTime);
					lpro.AddRelativeVelocity(Instigator.Velocity);
					// Touch any actor that was in between, just in case.
					if(HitActor != None)
						HitActor.Bump(lpro);
				}
			}
			else // fire a seeking rocket
			{
				// Shoot either a traditional seeking rocket or a new super seeker
				if(!bShootTradSeeker)
					lspro = spawn(class'kNukeMPSeekingProjectile',Instigator,,StartTrace, AdjustedAim);
				else
					lspro = spawn(class'kNukeMPSeekingProjectileTrad',Instigator,,StartTrace, AdjustedAim);

				if(lspro != None)
				{
					lspro.Instigator = Instigator;

					// if we're not aimed at anything, figure out a target
					if(CurrentTarget == None)
					{
						lspro.DetermineTarget(ProjectedHitLoc);
					}
					else
						lspro.SetNewTarget(CurrentTarget);

					lspro.SetupShot(SeekingChargeMultiplier*ChargeTime);

					lspro.AddRelativeVelocity(Instigator.Velocity);

					// Touch any actor that was in between, just in case.
					if(HitActor != None)
						HitActor.Bump(lspro);
				}
			}
		}

		// Clear our target regardless
		CurrentTarget = None;
		// If we didn't make a valid rocket, then give him his ammo back
		if(lpro == None
			&& lspro == None)
		{
			giveback = ChargeStartAmmo - AmmoType.AmmoAmount;
			if(giveback > 0)
				AmmoType.AddAmmo(giveback);
		}
		else
		{
			// Say we just fired
			ShotCount++;

			// Only make a new danger marker if the consecutive fires were as high
			// as the max
			if(ShotCount >= ShotCountMaxForNotify
				&& Instigator.Controller != None
				&& ShotMarkerMade != None)
			{
				// tell it we know this just happened, by recording it.
				ShotCount -= ShotCountMaxForNotify;
				
				// Records the first (gun fire)
				markerpos = Instigator.Location;
				
				// Primary (the gun shooting, making a loud noise)
				if(ShotMarkerMade != None)
				{
					ShotMarkerMade.static.NotifyControllersStatic(
						Level,
						ShotMarkerMade,
						FPSPawn(Instigator), 
						None, 
						ShotMarkerMade.default.CollisionRadius,
						markerpos);
				}
			}
		}
		// Reset the ammo you started with
		ChargeStartAmmo = AmmoType.AmmoAmount;
	}
}

state Active
{
	function BeginState()
	{
		Super.BeginState();
		P2Player(Instigator.Controller).ClientMessage("Nuke Launcher selected");
	}
}

defaultproperties {
	InitialAmmoCost=15
	GroupOffset=101
	PickupClass='kNukeMPPickup'
}