class kVoteHandler extends BroadcastHandler
	config(kMods);

struct VoteStruct
{
	var PlayerController pc;
	var string vote;
};

struct Tally
{
	var string map;
	var int votes;
};

struct BadWordStruct
{
	var string Word;
	var string Replace;
};

var bool bVoteInProgress;
var bool bTenSecondWarning;
var bool bVoteClosed;
var bool bExitingNow;
var string nextmap;
var string tiedmaps;
var kVoteReminder OurReminder;

var array<PlayerController> LlamaPlayers;
var array<PlayerController> MutePlayers;
var array<VoteStruct> CurrentVoters;
var array<string> Tiebreaker;

var config float VOTE_TIME;
var config float REVOTE_TIME;
var config float VOTE_PCT;
var config array<BadWordStruct> BadWords;

const ORGLE = "orgle";

static final function string ReplaceWith(coerce string Text, coerce string Replace, coerce string With)
{
    local int i;
    local string Output;
    
    i = InStr(Text, Replace);
    while (i != -1) {   
        Output = Output $ Left(Text, i) $ With;
        Text = Mid(Text, i + Len(Replace)); 
        i = InStr(Text, Replace);
    }
    Output = Output $ Text;
    return Output;
}

function SystemMessage( string Msg )
{
	local controller C;

	for( C=Level.ControllerList; C!=None; C=C.nextController )
		if( C.IsA('PlayerController') )
		{
			PlayerController(C).ClearProgressMessages();
			PlayerController(C).SetProgressTime(5);
			PlayerController(C).SetProgressMessage(0, Msg, class'Canvas'.Static.MakeColor(200,200,200));
		}
}

function string GetMapListAsString()
{
	local MapList MyList;
	local int i;
	local string s;

	s = "";
	MyList = Level.Game.GetMapList(Level.Game.MapListType);
	for ( i=0; i<ArrayCount(MyList.Maps); i++ )
		if (MyList.Maps[i] != "")
			s = s $ BaseMapName(MyList.Maps[i]) $ ", ";

	s = left(s, len(s)-2);

	return s;
}

function bool IsMuted(PlayerController P)
{
	local int i;

	for (i=0;i<LlamaPlayers.Length;i++)
		if (P == MutePlayers[i])
			return true;

	return false;
}

function bool IsLlamafied(PlayerController P)
{
	local int i;

	for (i=0;i<LlamaPlayers.Length;i++)
		if (P == LlamaPlayers[i])
			return true;

	return false;
}

function string Llamafy(string S)
{
	local int i,j;
	local string t;

	for (i=0;i<Len(s);i++)
		if((asc(mid(s,i,1)) >= asc("a") && asc(mid(s,i,1)) <= asc("z")) || (asc(mid(s,i,1)) >= asc("A") && asc(mid(s,i,1)) <= asc("Z")))
		{
			t = t $ mid(ORGLE,j,1);
			j++;
			if (j >= len(ORGLE))
				j = 0;
		}
		else
		{
			t = t $ mid(s,i,1);
			j = 0;
		}

	return t;
}

function bool AddMute(PlayerController pc)
{
	if (!IsMuted(pc))
	{
		MutePlayers.Insert(0,1);
		MutePlayers[0] = pc;
		return true;
	}

	return false;
}

function bool DelMute(PlayerController pc)
{
	local int i;

	for (i=0;i<MutePlayers.Length;i++)
		if (pc == MutePlayers[i])
		{
			MutePlayers.Remove(i,1);
			return true;
		}

	return false;
}

function bool AddLlama(PlayerController pc)
{
	if (!IsLlamafied(pc))
	{
		LlamaPlayers.Insert(0,1);
		LlamaPlayers[0] = pc;
		return true;
	}

	return false;
}

function bool DelLlama(PlayerController pc)
{
	local int i;

	for (i=0;i<LlamaPlayers.Length;i++)
		if (pc == LlamaPlayers[i])
		{
			LlamaPlayers.Remove(i,1);
			return true;
		}

	return false;
}

function int NumHumans()
{
	local int j;
	local Controller C;

	for (C=Level.Controllerlist; C!=None; C=C.NextController)
		if (PlayerController(C) != None)
			j++;

	return j;
}

function string BaseMapName(string s)
{
	local int i;
	local string t;
	t = s;
	i = InStr(t, "-");
	if (i > -1 && Left(t,2) ~= "MP")
		t = Right(t, Len(t)-i-1);
	if (Right(t, 4) ~= ".fuk")
		t = Left(t, Len(t)-4);

	//log ("it came out as " $ t);
	return t;
}

function string FullMapName(string s)
{
	local MapList MyList;
	local int i;
	local string t;

	MyList = Level.Game.GetMapList(Level.Game.MapListType);
	for ( i=0; i<ArrayCount(MyList.Maps); i++ )
		if ( s ~= BaseMapName(MyList.Maps[i]))
			return MyList.Maps[i];

	return "";
}

function bool MapExists(string s)
{
	local MapList MyList;
	local int i;
	local bool bExists;

	MyList = Level.Game.GetMapList(Level.Game.MapListType);
	for ( i=0; i<ArrayCount(MyList.Maps); i++ )
		if ( s ~= BaseMapName(MyList.Maps[i]))
			bExists=true;

	return bExists;
}

function bool IsATiebreaker(string s)
{
	local int i;
	for (i=0;i<Tiebreaker.Length;i++)
		if (s ~= Tiebreaker[i])
			return true;

	return false;
}

function AddVote(PlayerController pc, string s)
{
	local int i, j, k, VotesReq;
	local bool bAlreadyVoted;

	if (Tiebreaker.Length > 0)
		if (!IsATiebreaker(s))
		{
			pc.ClientMessage("ERROR: Tiebreaker round, please vote for one of the following maps: " $ tiedmaps);
			return;
		}

	if (!MapExists(s))
	{
		pc.ClientMessage("ERROR: Map '" $ s $ "' doesn't exist in the current rotation. Say 'maplist' for a list of maps");
		return;
	}

	i = CurrentVoters.Length;
	for (k=0;k<CurrentVoters.Length;k++)
		if (CurrentVoters[k].pc == pc)
		{
			CurrentVoters[k].vote = s;
			bAlreadyVoted=true;
		}

	if (!bAlreadyVoted)
	{
		CurrentVoters.Insert(i,1);
		CurrentVoters[i].pc = pc;
		CurrentVoters[i].vote = s;
	}

	j = NumHumans() * VOTE_PCT;
	VotesReq=(j - i - 1);
	if (VotesReq<0)
		VotesReq = 0;
	if (s != "")
		Broadcast(Self, pc.PlayerReplicationInfo.PlayerName @ "votes for" @ s $ "," @ VotesReq @ "more votes required to exit");
	else
		Broadcast(Self, pc.PlayerReplicationInfo.PlayerName @ "votes to exit," @ VotesReq @ "more votes required to exit");
}

function Timer()
{
	local string winner;
	local int winvotes, VotesReq;
	local bool bTie;
	local int i,j;
	local array<Tally> Finalists;
	local bool bVoteExists;

	if (bExitingNow)
	{
		if (nextmap != "")
			Level.ServerTravel(nextmap, false);
	
		Level.Game.bChangeLevels=true;
		Level.Game.RestartGame();
		return;
	}

	if (bVoteClosed)
	{
		bVoteClosed=false;
		return;
	}

	if (!bTenSecondWarning)
	{
		SystemMessage("10 seconds left to place your vote!");
		bTenSecondWarning=true;
		SetTimer(10, false);
		return;
	}

	bTenSecondWarning=false;

	bVoteInProgress=false;

	OurReminder.Destroy();

	i = CurrentVoters.Length;
	j = NumHumans() * VOTE_PCT;
	VotesReq=(j - i);
	if (VotesReq>0)
	{
		SystemMessage("Not enough votes, exit vote cancelled");
		bVoteClosed=true;
		SetTimer(REVOTE_TIME, false);
		return;
	}

	if (Finalists.Length>0)
		Finalists.Remove(0, Finalists.Length);
	
	for (i=0;i<CurrentVoters.Length;i++)
	{
		bVoteExists=false;
		for (j=0;j<Finalists.Length;j++)
		{
			if (Finalists[j].map ~= CurrentVoters[i].vote && CurrentVoters[i].Vote != "")
			{
				bVoteExists=true;
				Finalists[j].votes++;
			}
		}
		if (!bVoteExists && CurrentVoters[i].Vote != "")
		{
			Finalists.Insert(Finalists.Length, 1);
			Finalists[Finalists.Length-1].map = CurrentVoters[i].vote;
			Finalists[Finalists.Length-1].votes = 1;
		}
	}

	for (i=0;i<Finalists.Length;i++)
	{
		if (Finalists[i].Votes > winvotes)
		{
			bTie=false;
			winvotes = Finalists[i].Votes;
			winner = Finalists[i].Map;
			if (Tiebreaker.Length>0)
				Tiebreaker.Remove(0, Tiebreaker.Length);
			Tiebreaker.Insert(0, 1);
			Tiebreaker[0] = winner;
		}
		else if (Finalists[i].Votes == winvotes)
		{
			Tiebreaker.Insert(Tiebreaker.Length, 1);
			Tiebreaker[Tiebreaker.Length-1] = Finalists[i].Map;
		}
	}
	if (Tiebreaker.Length>1)
	{
		tiedmaps="[";
		for (i=0;i<Tiebreaker.Length;i++)
			tiedmaps = tiedmaps $ Tiebreaker[i] $ ",";
		tiedmaps=Left(tiedmaps, Len(tiedmaps)-1) $ "]";
		SystemMessage("TIEBREAKER VOTE!");
		Broadcast(self,"Vote for one of the following maps only: " $ tiedmaps);
		bVoteInProgress=true;
		SetTimer(VOTE_TIME - 10, false);
		if (CurrentVoters.Length>0)
			CurrentVoters.Remove(0, CurrentVoters.Length);
		return;
	}

	nextmap=FullMapName(winner);
	SystemMessage("Voting has ended! Switching maps in 10 seconds...");
	bExitingNow=true;
	SetTimer(10, false);
}

function Broadcast( Actor Sender, coerce string Msg, optional name Type )
{
	local int i;
	local string s;
	local PlayerController pc;

	//Log(Sender @ "said" @ Msg @ "and i was" @ i);

	pc=PlayerController(Sender);
	if (pc != None)
	{
		if (left(Msg,11) ~= "rockthevote" && !bVoteInProgress && !bVoteClosed)
		{
			if (Tiebreaker.Length>0)
				Tiebreaker.Remove(0, Tiebreaker.Length);
			SystemMessage(pc.PlayerReplicationInfo.PlayerName @ "has rocked the vote!");
			Broadcast(self,"Exit vote open! If you want to exit, say 'vote (mapname)' or just plain 'vote'");
			bVoteInProgress=true;
			if (CurrentVoters.Length>0)
				CurrentVoters.Remove(0, CurrentVoters.Length);
			SetTimer(VOTE_TIME - 10, false);
			OurReminder = spawn(class'kVoteReminder');
			OurReminder.SetTimer(VOTE_TIME/2, false);
			return;
		}
	
		if (left(Msg,4) ~= "vote" && bVoteInProgress)
		{
			if (Len(Msg)>4)
			{
				s = Right(Msg, Len(Msg) - 5);
				if (s ~= "exit")
					s="";
			}
			AddVote(pc, s);
			return;
		}

		if (left(Msg,7) ~= "maplist")
		{
			s = "Maps in rotation:" @ GetMapListAsString();
			//log ("Telling" @ pc @ "that the maplist is" @ s);
			pc.ClientMessage(s);
			return;
		}

		if (IsLlamafied(pc))
			Msg=Llamafy(Msg);

		if (IsMuted(pc))
			return;
	}

	for (i=0;i<BadWords.Length;i++)
		Msg = ReplaceWith(Msg, BadWords[i].Word, BadWords[i].Replace);

	Super.Broadcast(Sender, Msg, Type);
}

defaultproperties
{
	VOTE_TIME=60;
	REVOTE_TIME=240;
	VOTE_PCT=0.5;
	BadWords[0]=(Word=fuck,Replace=fridge);
	BadWords[1]=(Word=faggot,Replace=best friend);
	BadWords[2]=(Word=fag,Replace=friend);
}